#include "StdAfx.h"
#include "ContainerScriptletPlugin.h"
#include "XMLUtil.h"
#include "ScriptletConstant.h"
#include "Container.h"
#include "ScriptletContainerFactory.h"
#include "Application.h"
#include "ScriptletContainerFactory.h"

CContainerScriptletPlugin::CContainerScriptletPlugin(void)
{
}

CContainerScriptletPlugin::~CContainerScriptletPlugin(void)
{
}
BOOL CContainerScriptletPlugin::ModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	BOOL bSuccess = __super::ModifyComponent(pParent, pTarget, pNode);
	if (bSuccess)
	{
		CContainer* pContainer = reinterpret_cast<CContainer*>(pTarget);
		if (pContainer == NULL)
		{
			return FALSE;
		}		
		
		TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
		long lValue = 0;
		USES_CONVERSION;
		// layout
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_LAYOUT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_LAYOUT_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			ILayoutManager* pLayout = CScriptletContainerFactory::get_instance()->CreateLayout(szValue);
			if ( NULL != pLayout )
			{
				pContainer->SetLayoutManager(pLayout);
				pLayout->AutoRelease();
			}
		}
		//valign
		// layout
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_VALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_VALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			if (_tcsicmp(szValue, SCRIPTLET_TOP_VALUE) == 0)
			{
				pContainer->SetVerticalAlignment(VerticalAlignment::Top);
			}
			else if (_tcsicmp(szValue, SCRIPTLET_CENTER_VALUE) == 0)
			{
				pContainer->SetVerticalAlignment(VerticalAlignment::Center);
			}
			else if (_tcsicmp(szValue, SCRIPTLET_BOTTOM_VALUE) == 0)
			{
				pContainer->SetVerticalAlignment(VerticalAlignment::Bottom);
			}
			else
			{
				pContainer->SetVerticalAlignment(VerticalAlignment::Stretch);
			}
		}
		//halign
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_HALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_HALIGN_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			if (_tcsicmp(szValue, SCRIPTLET_LEFT_VALUE) == 0)
			{
				pContainer->SetHorizontalAlignment(HorizonAlignment::Left);
			}
			else if (_tcsicmp(szValue, SCRIPTLET_CENTER_VALUE) == 0)
			{
				pContainer->SetHorizontalAlignment(HorizonAlignment::Center);
			}
			else if (_tcsicmp(szValue, SCRIPTLET_RIGHT_VALUE) == 0)
			{
				pContainer->SetHorizontalAlignment(HorizonAlignment::Right);
			}
			else
			{
				pContainer->SetHorizontalAlignment(HorizonAlignment::Stretch);
			}
		}
		//margin
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_MARGIN_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_MARGIN_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			RECT rcMargin = {0};
			CString strThickness = szValue;
			int nToken = 0;
			rcMargin.left = _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
			if (nToken > _tcslen(szValue))
			{
				rcMargin.top = rcMargin.left;
				rcMargin.right = rcMargin.left;
				rcMargin.bottom = rcMargin.top;
			}
			else
			{
				rcMargin.top = _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
				if (nToken > _tcslen(szValue))
				{
					rcMargin.right = rcMargin.left;
					rcMargin.bottom = rcMargin.top;
				}
				else
				{
					rcMargin.right =  _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
					if (nToken > _tcslen(szValue))
					{
						rcMargin.bottom = rcMargin.top;
					}
					else
					{
						rcMargin.bottom = _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
					}
				}
			}
			pContainer->SetMargin(&rcMargin);
		}	
		//padding
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_PADDING_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_PADDING_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			RECT rcPadding = {0};
			CString strThickness = szValue;
			int nToken = 0;
			rcPadding.left = _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
			if (nToken > _tcslen(szValue))
			{
				rcPadding.top = rcPadding.left;
				rcPadding.right = rcPadding.left;
				rcPadding.bottom = rcPadding.top;
			}
			else
			{
				rcPadding.top = _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
				if (nToken > _tcslen(szValue))
				{
					rcPadding.right = rcPadding.left;
					rcPadding.bottom = rcPadding.top;
				}
				else
				{
					rcPadding.right =  _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
					if (nToken > _tcslen(szValue))
					{
						rcPadding.bottom = rcPadding.top;
					}
					else
					{
						rcPadding.bottom = _ttoi(strThickness.Tokenize(SCRIPTLET_SEPARATOR, nToken));
					}
				}
			}
			pContainer->SetPadding(&rcPadding);
		}
		//datacontext
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_DATACONTEXT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_DATACONTEXT_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			IDataContext* context = IDataContext::FindDataContext(szValue);
			//IDataContext* context = CScriptletContainerFactory::get_instance()->FindDataContext(szValue);
			if ( NULL != context )
			{
				pContainer->SetDataContext(context);				
			}
		}
		CreateChildComponent(pNode, pContainer);		
	}	
	return bSuccess;
}
void CContainerScriptletPlugin::CreateChildComponent( TiXmlElement* pNode, CContainer* pContainer )
{
	TiXmlElement* pChildNode = pNode->FirstChildElement();
	while (pChildNode)
	{
		CComponent* pChildComponent = CScriptletContainerFactory::get_instance()->CreateComponent(pContainer, pChildNode);
		pChildNode = pChildNode->NextSiblingElement();
	}
	//pContainer->RecalcLayout();
}